/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.fields;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * ファイルやリソースを選択するための抽象テキスト・フィールドです。
 */
public abstract class AbstractBrowseTextField extends TextField implements IBrowseTextField {

	protected static final Map<String, String> previousInputMap = new HashMap<String, String>();

	protected final Button button;
	protected final String[] extensions;

	public AbstractBrowseTextField(Composite parent, String labelText, String... extensions) {
		super(parent, labelText);
		initializeDialogUnits(parent);

		final String prevDir = previousInputMap.get(labelText);
		text.setText(prevDir == null ? "" : prevDir);

		button = new Button(parent, SWT.PUSH);
		button.setText("参照...");
		button.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				openDialog();
			}
		});
		setButtonLayoutData(button);

		this.extensions = new String[extensions.length];
		for (int i = 0; i < extensions.length; i++) {
			this.extensions[i] = "*" + extensions[i];
		}
	}

	@Override
	public void setText(String path) {
		path = resolvePath(path);
		if (new File(path).exists()) {
			super.setText(path);
		} else {
			final String prevDir = previousInputMap.get(labelText);
			super.setText(prevDir);
		}
	}

	@Override
	public String getText() {
		final String path = super.getText();
		if (path.length() > 0) {
			previousInputMap.put(labelText, path);
		}
		return path;
	}

	public File getFile() {
		return new File(getText());
	}

	abstract protected void openDialog();

	protected String resolvePath(String path) {
		return path;
	}

	public boolean invalidExtension() {
		if (extensions.length == 0) {
			return false;
		}
		final String path = super.getText();
		if (path == null || path.equals("")) {
			return false;
		}
		for (final String extension : extensions) {
			final String ext = extension.replace("*", "");
			if (path.endsWith(ext)) {
				return false;
			}
		}
		return true;
	}

	// -------------------------------------------------------------------------
	// org.eclipse.jface.dialogs.DialogPage のコピー

	private FontMetrics fontMetrics;

	protected GridData setButtonLayoutData(Button button) {
		final GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		final int widthHint = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
		final Point minSize = button.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
		data.widthHint = Math.max(widthHint, minSize.x);
		button.setLayoutData(data);
		return data;
	}

	protected int convertHorizontalDLUsToPixels(int dlus) {
		if (fontMetrics == null) {
			return 0;
		}
		return Dialog.convertHorizontalDLUsToPixels(fontMetrics, dlus);
	}

	protected void initializeDialogUnits(Control testControl) {
		final GC gc = new GC(testControl);
		gc.setFont(JFaceResources.getDialogFont());
		fontMetrics = gc.getFontMetrics();
		gc.dispose();
	}
}
